import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { ChangeDocEvent } from 'src/modules/configuration/couch/domain/events/change-doc.event';
import { TransactionType } from '../../../constants';
import { TransactionDataService } from '../../../data/services/transaction-data.service';
import { TaxDataService } from 'src/modules/transaction/tax/data/services/tax-data.service';
import { SalesPriceFormulaDataService } from 'src/modules/transaction/sales-price-formula/data/services/sales-price-formula-data.service';
import { FormulaType } from 'src/modules/transaction/sales-price-formula/constants';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { TransactionModel } from '../../../data/models/transaction.model';
import { mappingRevertTransaction } from '../managers/helpers/mapping-transaction.helper';

@EventsHandler(ChangeDocEvent)
export class PosTransactionHandler implements IEventHandler<ChangeDocEvent> {
  constructor(
    private dataService: TransactionDataService,
    private taxService: TaxDataService,
    private formulaService: SalesPriceFormulaDataService,
  ) {}

  async handle(event: ChangeDocEvent) {
    try {
      const database = event.data.database;
      const data = event.data.data;

      // jika bukan database transaksi, return langsung
      if (database != 'transaction') return;

      const sales_formula = await this.formulaService.getOneByOptions({
        where: {
          type: FormulaType.SALES_PRICE,
        },
      });

      const profit_formula = await this.formulaService.getOneByOptions({
        where: {
          type: FormulaType.PROFIT_SHARE,
        },
      });

      const taxes = await this.taxService.getManyByOptions({
        where: {
          status: STATUS.ACTIVE,
        },
      });

      const queryRunner = this.dataService
        .getRepository()
        .manager.connection.createQueryRunner();

      // jika delete
      if (data._deleted ?? false) {
        await this.dataService.deleteById(
          queryRunner,
          TransactionModel,
          data._id,
        );
      }

      // jika update // create
      else {
        const tax_datas = taxes?.map((tax) => {
          return {
            tax_id: tax.id,
            tax_name: tax.name,
            tax_value: tax.value,
          };
        });

        mappingRevertTransaction(data, TransactionType.COUNTER);

        Object.assign(data, {
          taxes: tax_datas,
          profit_share_formula: profit_formula.formula_string,
          sales_price_formula: sales_formula.formula_string,
        });

        await this.dataService.create(queryRunner, TransactionModel, data);
      }
    } catch (error) {
      console.log('error handling pos transaction couch');
    }
  }
}
